@@ -1,5 +1,5 @@
 /* Print usage information and help for ld.so.
-   Copyright (C) 1995-2022 Free Software Foundation, Inc.
+   Copyright (C) 1995-2024 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    The GNU C Library is free software; you can redistribute it and/or
@@ -41,7 +41,7 @@ _dl_version (void)
 {
   _dl_printf ("\
 ld.so " PKGVERSION RELEASE " release version " VERSION ".\n\
-Copyright (C) 2022 Free Software Foundation, Inc.\n\
+Copyright (C) 2024 Free Software Foundation, Inc.\n\
 This is free software; see the source for copying conditions.\n\
 There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A\n\
 PARTICULAR PURPOSE.\n\
@@ -99,39 +99,12 @@ print_search_path_for_help (struct dl_main_state *state)
   if (map != NULL)
     print_search_path_for_help_1 (map->l_runpath_dirs.dirs);
 
-  _dl_printf ("  (libraries located via %s)\n", LD_SO_CACHE);
+  if (!GLRO(dl_inhibit_cache))
+    _dl_printf ("  (libraries located via %s)\n", LD_SO_CACHE);
 
   print_search_path_for_help_1 (__rtld_search_dirs.dirs);
 }
 
-/* Helper function for printing flags associated with a HWCAP name.  */
-static void
-print_hwcap_1 (bool *first, bool active, const char *label)
-{
-  if (active)
-    {
-      if (*first)
-        {
-          _dl_printf (" (");
-          *first = false;
-        }
-      else
-        _dl_printf (", ");
-      _dl_printf ("%s", label);
-    }
-}
-
-/* Called after a series of print_hwcap_1 calls to emit the line
-   terminator.  */
-static void
-print_hwcap_1_finish (bool *first)
-{
-  if (*first)
-    _dl_printf ("\n");
-  else
-    _dl_printf (")\n");
-}
-
 /* Print the header for print_hwcaps_subdirectories.  */
 static void
 print_hwcaps_subdirectories_header (bool *nothing_printed)
@@ -165,9 +138,7 @@ print_hwcaps_subdirectories (const struct dl_main_state *state)
     {
       print_hwcaps_subdirectories_header (&nothing_printed);
       print_hwcaps_subdirectories_name (&split);
-      bool first = true;
-      print_hwcap_1 (&first, true, "searched");
-      print_hwcap_1_finish (&first);
+      _dl_printf (" (searched)\n");
     }
 
   /* The built-in glibc-hwcaps subdirectories.  Do the filtering
@@ -178,13 +149,14 @@ print_hwcaps_subdirectories (const struct dl_main_state *state)
     {
       print_hwcaps_subdirectories_header (&nothing_printed);
       print_hwcaps_subdirectories_name (&split);
-      bool first = true;
-      print_hwcap_1 (&first, mask & 1, "supported");
       bool listed = _dl_hwcaps_contains (state->glibc_hwcaps_mask,
                                          split.segment, split.length);
-      print_hwcap_1 (&first, !listed, "masked");
-      print_hwcap_1 (&first, (mask & 1) && listed, "searched");
-      print_hwcap_1_finish (&first);
+      if (mask & 1)
+        _dl_printf (" (supported, %s)\n", listed ? "searched" : "masked");
+      else if (!listed)
+        _dl_printf (" (masked)\n");
+      else
+        _dl_printf ("\n");
       mask >>= 1;
     }
 
@@ -193,37 +165,6 @@ print_hwcaps_subdirectories (const struct dl_main_state *state)
 No subdirectories of glibc-hwcaps directories are searched.\n");
 }
 
-/* Write a list of hwcap subdirectories to standard output.  See
- _dl_important_hwcaps in dl-hwcaps.c.  */
-static void
-print_legacy_hwcap_directories (void)
-{
-  _dl_printf ("\n\
-Legacy HWCAP subdirectories under library search path directories:\n");
-
-  const char *platform = GLRO (dl_platform);
-  if (platform != NULL)
-    _dl_printf ("  %s (AT_PLATFORM; supported, searched)\n", platform);
-
-  _dl_printf ("  tls (supported, searched)\n");
-
-  uint64_t hwcap_mask = GET_HWCAP_MASK();
-  uint64_t searched = GLRO (dl_hwcap) & hwcap_mask;
-  for (int n = 63; n >= 0; --n)
-    {
-      uint64_t bit = 1ULL << n;
-      if (HWCAP_IMPORTANT & bit)
-        {
-          _dl_printf ("  %s", _dl_hwcap_string (n));
-          bool first = true;
-          print_hwcap_1 (&first, GLRO (dl_hwcap) & bit, "supported");
-          print_hwcap_1 (&first, !(hwcap_mask & bit), "masked");
-          print_hwcap_1 (&first, searched & bit, "searched");
-          print_hwcap_1_finish (&first);
-        }
-    }
-}
-
 void
 _dl_help (const char *argv0, struct dl_main_state *state)
 {
@@ -255,12 +196,8 @@ setting environment variables (which would be inherited by subprocesses).\n\
                         in LIST\n\
   --audit LIST          use objects named in LIST as auditors\n\
   --preload LIST        preload objects named in LIST\n\
-  --argv0 STRING        set argv[0] to STRING before running\n"
-#if HAVE_TUNABLES
-"\
-  --list-tunables       list all tunables with minimum and maximum values\n"
-#endif
-"\
+  --argv0 STRING        set argv[0] to STRING before running\n\
+  --list-tunables       list all tunables with minimum and maximum values\n\
   --list-diagnostics    list diagnostics information\n\
   --help                display this help and exit\n\
   --version             output version information and exit\n\
@@ -270,6 +207,5 @@ This program interpreter self-identifies as: " RTLD "\n\
               argv0);
   print_search_path_for_help (state);
   print_hwcaps_subdirectories (state);
-  print_legacy_hwcap_directories ();
   _exit (EXIT_SUCCESS);
 }
