@@ -1,5 +1,5 @@
 /* Get calling thread's ID.
-   Copyright (C) 2000-2023 Free Software Foundation, Inc.
+   Copyright (C) 2000-2024 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    The GNU C Library is free software; you can redistribute it and/or
@@ -17,23 +17,20 @@
    <https://www.gnu.org/licenses/>.  */
 
 #include <pthread.h>
-
+#include <shlib-compat.h>
 #include <pt-internal.h>
 
 /* Return the thread ID of the calling thread.  */
 pthread_t
 __pthread_self (void)
 {
-  struct __pthread *self;
-
-  if (___pthread_self == NULL)
-    /* We are not initialized yet, we are the first thread.  */
-    return 1;
-
-  self = _pthread_self ();
-  assert (self != NULL);
-
+  struct __pthread *self = _pthread_self ();
   return self->thread;
 }
 
-weak_alias (__pthread_self, pthread_self);
+libc_hidden_def (__pthread_self)
+versioned_symbol (libc, __pthread_self, pthread_self, GLIBC_2_21);
+
+#if OTHER_SHLIB_COMPAT (libpthread, GLIBC_2_12, GLIBC_2_21)
+compat_symbol (libc, __pthread_self, pthread_self, GLIBC_2_12);
+#endif
