@@ -1,5 +1,5 @@
 /* Set thread_state for sighandler, and sigcontext to recover.  Stub version.
-   Copyright (C) 1994-2016 Free Software Foundation, Inc.
+   Copyright (C) 1994-2024 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    The GNU C Library is free software; you can redistribute it and/or
@@ -14,7 +14,7 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with the GNU C Library; if not, see
-   <http://www.gnu.org/licenses/>.  */
+   <https://www.gnu.org/licenses/>.  */
 
 #include <hurd.h>
 #include <mach/thread_status.h>
@@ -26,11 +26,11 @@
    that structure.  */
 
 struct sigcontext *
-_hurd_setup_sighandler (int flags,
-			__sighandler_t handler,
-			struct sigaltstack *sigaltstack,
-			int signo, int sigcode,
-			void *state)
+_hurd_setup_sighandler (struct hurd_sigstate *ss, const struct sigaction *action,
+                        __sighandler_t handler,
+                        int signo, struct hurd_signal_detail *detail,
+                        int rpc_wait,
+                        struct machine_thread_all_state *state)
 {
 #error "Need to write sysdeps/mach/hurd/MACHINE/trampoline.c"
 }
