@@ -1,4 +1,4 @@
-/* Copyright (C) 1995-2022 Free Software Foundation, Inc.
+/* Copyright (C) 1995-2024 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    The GNU C Library is free software; you can redistribute it and/or
@@ -24,41 +24,53 @@
    This exception applies to code released by its copyright holders
    in files containing the exception.  */
 
-#include <libioP.h>
+#include <array_length.h>
+#include <math_ldbl_opt.h>
+#include <printf.h>
 #include <stdio_ext.h>
+#include <unistd.h>
+#include <printf_buffer.h>
+
+struct __printf_buffer_dprintf
+{
+  struct __printf_buffer base;
+  int fd;
+
+  char buf[PRINTF_BUFFER_SIZE_DPRINTF];
+};
+
+void
+__printf_buffer_flush_dprintf (struct __printf_buffer_dprintf *buf)
+{
+  char *p = buf->buf;
+  char *end = buf->base.write_ptr;
+  while (p < end)
+    {
+      ssize_t ret = TEMP_FAILURE_RETRY (write (buf->fd, p, end - p));
+      if (ret < 0)
+	{
+	  __printf_buffer_mark_failed (&buf->base);
+	  return;
+	}
+      p += ret;
+    }
+  buf->base.written += buf->base.write_ptr - buf->base.write_base;
+  buf->base.write_ptr = buf->buf;
+}
 
 int
 __vdprintf_internal (int d, const char *format, va_list arg,
 		     unsigned int mode_flags)
 {
-  struct _IO_FILE_plus tmpfil;
-  struct _IO_wide_data wd;
-  int done;
-
-#ifdef _IO_MTSAFE_IO
-  tmpfil.file._lock = NULL;
-#endif
-  _IO_no_init (&tmpfil.file, _IO_USER_LOCK, 0, &wd, &_IO_wfile_jumps);
-  _IO_JUMPS (&tmpfil) = &_IO_file_jumps;
-  _IO_new_file_init_internal (&tmpfil);
-  if (_IO_file_attach (&tmpfil.file, d) == NULL)
-    {
-      _IO_un_link (&tmpfil);
-      return EOF;
-    }
-  tmpfil.file._flags |= _IO_DELETE_DONT_CLOSE;
-
-  _IO_mask_flags (&tmpfil.file, _IO_NO_READS,
-		  _IO_NO_READS+_IO_NO_WRITES+_IO_IS_APPENDING);
-
-  done = __vfprintf_internal (&tmpfil.file, format, arg, mode_flags);
-
-  if (done != EOF && _IO_do_flush (&tmpfil.file) == EOF)
-    done = EOF;
-
-  _IO_FINISH (&tmpfil.file);
-
-  return done;
+  struct __printf_buffer_dprintf buf;
+  __printf_buffer_init (&buf.base, buf.buf, array_length (buf.buf),
+			__printf_buffer_mode_dprintf);
+  buf.fd = d;
+  __printf_buffer (&buf.base, format, arg, mode_flags);
+  if (__printf_buffer_has_failed (&buf.base))
+    return -1;
+  __printf_buffer_flush_dprintf (&buf);
+  return __printf_buffer_done (&buf.base);
 }
 
 int
