@@ -1,5 +1,5 @@
 /* Verify that fseek/ftell combination works for wide chars.
-   Copyright (C) 2012-2015 Free Software Foundation, Inc.
+   Copyright (C) 2012-2024 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    The GNU C Library is free software; you can redistribute it and/or
@@ -14,7 +14,7 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with the GNU C Library; if not, see
-   <http://www.gnu.org/licenses/>.  */
+   <https://www.gnu.org/licenses/>.  */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -24,9 +24,7 @@
 #include <unistd.h>
 #include <string.h>
 
-/* Defined in test-skeleton.c.  */
-static int create_temp_file (const char *base, char **filename);
-
+#include <support/temp_file.h>
 
 static int
 do_seek_end (FILE *fp)
@@ -168,6 +166,4 @@ do_test (void)
   return ret;
 }
 
-
-#define TEST_FUNCTION do_test ()
-#include "../test-skeleton.c"
+#include <support/test-driver.c>
