@@ -1,4 +1,4 @@
-/* Copyright (C) 1998-2023 Free Software Foundation, Inc.
+/* Copyright (C) 1998-2024 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    The GNU C Library is free software; you can redistribute it and/or
@@ -200,21 +200,21 @@ special (void)
     output_error (NAME (ECVT), INFINITY, 10, "inf", 0, 0, p, decpt, sign);
 
   /* Simply make sure these calls with large NDIGITs don't crash.  */
-  (void) ECVT (123.456, 10000, &decpt, &sign);
-  (void) FCVT (123.456, 10000, &decpt, &sign);
+  p = ECVT (123.456, 10000, &decpt, &sign);
+  p = FCVT (123.456, 10000, &decpt, &sign);
 
   /* Some tests for the reentrant functions.  */
   /* Use a too small buffer.  */
   res = ECVT_R (123.456, 10, &decpt, &sign, buf, 1);
   if (res == 0)
     {
-      printf (NAME (ECVT_R) " with a too small buffer was succesful.\n");
+      printf (NAME (ECVT_R) " with a too small buffer was successful.\n");
       support_record_failure ();
     }
   res = FCVT_R (123.456, 10, &decpt, &sign, buf, 1);
   if (res == 0)
     {
-      printf (NAME (FCVT_R) " with a too small buffer was succesful.\n");
+      printf (NAME (FCVT_R) " with a too small buffer was successful.\n");
       support_record_failure ();
     }
 }
