@@ -1,6 +1,5 @@
-/* Copyright (C) 2002-2021 Free Software Foundation, Inc.
+/* Copyright (C) 2002-2024 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
-   Contributed by Ulrich Drepper <drepper@redhat.com>, 2002.
 
    The GNU C Library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -17,12 +16,19 @@
    <https://www.gnu.org/licenses/>.  */
 
 #include "pthreadP.h"
-
+#include <shlib-compat.h>
 
 int
-pthread_rwlockattr_getkind_np (const pthread_rwlockattr_t *attr, int *pref)
+__pthread_rwlockattr_getkind_np (const pthread_rwlockattr_t *attr, int *pref)
 {
   *pref = ((const struct pthread_rwlockattr *) attr)->lockkind;
 
   return 0;
 }
+versioned_symbol (libc, __pthread_rwlockattr_getkind_np,
+                  pthread_rwlockattr_getkind_np, GLIBC_2_34);
+
+#if OTHER_SHLIB_COMPAT (libpthread, GLIBC_2_1, GLIBC_2_34)
+compat_symbol (libpthread, __pthread_rwlockattr_getkind_np,
+               pthread_rwlockattr_getkind_np, GLIBC_2_1);
+#endif
