@@ -1,6 +1,5 @@
-/* Copyright (C) 2002-2018 Free Software Foundation, Inc.
+/* Copyright (C) 2002-2024 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
-   Contributed by Ulrich Drepper <drepper@redhat.com>, 2002.
 
    The GNU C Library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -14,15 +13,23 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with the GNU C Library; if not, see
-   <http://www.gnu.org/licenses/>.  */
+   <https://www.gnu.org/licenses/>.  */
 
 #include "pthreadP.h"
-
+#include <shlib-compat.h>
 
 int
-pthread_rwlockattr_getpshared (const pthread_rwlockattr_t *attr, int *pshared)
+__pthread_rwlockattr_getpshared (const pthread_rwlockattr_t *attr,
+                                  int *pshared)
 {
   *pshared = ((const struct pthread_rwlockattr *) attr)->pshared;
 
   return 0;
 }
+versioned_symbol (libc, __pthread_rwlockattr_getpshared,
+                  pthread_rwlockattr_getpshared, GLIBC_2_34);
+
+#if OTHER_SHLIB_COMPAT (libpthread, GLIBC_2_1, GLIBC_2_34)
+compat_symbol (libpthread, __pthread_rwlockattr_getpshared,
+               pthread_rwlockattr_getpshared, GLIBC_2_1);
+#endif
