@@ -121,7 +121,9 @@ static const struct conf vars[] =
     { "WORD_BIT", _SC_WORD_BIT, SYSCONF },
     { "_AVPHYS_PAGES", _SC_AVPHYS_PAGES, SYSCONF },
     { "_NPROCESSORS_CONF", _SC_NPROCESSORS_CONF, SYSCONF },
+    { "NPROCESSORS_CONF", _SC_NPROCESSORS_CONF, SYSCONF },
     { "_NPROCESSORS_ONLN", _SC_NPROCESSORS_ONLN, SYSCONF },
+    { "NPROCESSORS_ONLN", _SC_NPROCESSORS_ONLN, SYSCONF },
     { "_PHYS_PAGES", _SC_PHYS_PAGES, SYSCONF },
     { "_POSIX_ARG_MAX", _SC_ARG_MAX, SYSCONF },
     { "_POSIX_ASYNCHRONOUS_IO", _SC_ASYNCHRONOUS_IO, SYSCONF },
