@@ -1,5 +1,5 @@
 /* POSIX.2 wordexp implementation.
-   Copyright (C) 1997-2023 Free Software Foundation, Inc.
+   Copyright (C) 1997-2024 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    The GNU C Library is free software; you can redistribute it and/or
@@ -720,7 +720,7 @@ parse_arith (char **word, size_t *word_length, size_t *max_length,
 	      ++(*offset);
 
 	      /* Go - evaluate. */
-	      if (*expr && eval_expr (expr, &numresult) != 0)
+	      if (expr && eval_expr (expr, &numresult) != 0)
 		{
 		  free (expr);
 		  return WRDE_SYNTAX;
@@ -758,7 +758,7 @@ parse_arith (char **word, size_t *word_length, size_t *max_length,
 	      long int numresult = 0;
 
 	      /* Go - evaluate. */
-	      if (*expr && eval_expr (expr, &numresult) != 0)
+	      if (expr && eval_expr (expr, &numresult) != 0)
 		{
 		  free (expr);
 		  return WRDE_SYNTAX;
@@ -817,8 +817,8 @@ exec_comm_child (char *comm, int *fildes, bool showerr, bool noexec)
   __posix_spawn_file_actions_init (&fa);
 
   /* Redirect output.  For check syntax only (noexec being true), exec_comm
-     explicits sets fildes[1] to -1, so check its value to avoid a failure in
-     __posix_spawn_file_actions_adddup2.  */
+     explicitly sets fildes[1] to -1, so check its value to avoid a failure
+     in __posix_spawn_file_actions_adddup2.  */
   if (fildes[1] != -1)
     {
       if (__glibc_likely (fildes[1] != STDOUT_FILENO))
@@ -1790,7 +1790,7 @@ envsubst:
 	    {
 	      const char *str = pattern;
 
-	      if (str[0] == '\0')
+	      if (!str || str[0] == '\0')
 		str = _("parameter null or not set");
 
 	      __fxprintf (NULL, "%s: %s\n", env, str);
@@ -1883,10 +1883,7 @@ envsubst:
 			_itoa_word (value ? strlen (value) : 0,
 				    &param_length[20], 10, 0));
       if (free_value)
-	{
-	  assert (value != NULL);
-	  free (value);
-	}
+	free (value);
 
       return *word ? 0 : WRDE_NOSPACE;
     }
