@@ -20,8 +20,8 @@
 #define UNSIGNED	1
 
 #include <features.h>
-#undef __GLIBC_USE_C2X_STRTOL
-#define __GLIBC_USE_C2X_STRTOL 0
+#undef __GLIBC_USE_C23_STRTOL
+#define __GLIBC_USE_C23_STRTOL 0
 #include <locale.h>
 #include <stdbool.h>
 
