@@ -1,8 +1,7 @@
 /* Round argument to nearest integral value according to current rounding
    direction.
-   Copyright (C) 1997-2019 Free Software Foundation, Inc.
+   Copyright (C) 1997-2024 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
-   Contributed by Ulrich Drepper <drepper@cygnus.com>, 1997.
 
    The GNU C Library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -16,7 +15,7 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with the GNU C Library; if not, see
-   <http://www.gnu.org/licenses/>.  */
+   <https://www.gnu.org/licenses/>.  */
 
 #include <fenv.h>
 #include <limits.h>
@@ -26,17 +25,22 @@
 #include <math_private.h>
 #include <libm-alias-double.h>
 #include <fix-fp-int-convert-overflow.h>
-
-static const double two52[2] =
-{
-  4.50359962737049600000e+15, /* 0x43300000, 0x00000000 */
- -4.50359962737049600000e+15, /* 0xC3300000, 0x00000000 */
-};
+#include <math-use-builtins.h>
 
 
 long long int
 __llrint (double x)
 {
+#if USE_LLRINT_BUILTIN
+  return __builtin_llrint (x);
+#else
+  /* Use generic implementation.  */
+  static const double two52[2] =
+  {
+    4.50359962737049600000e+15, /* 0x43300000, 0x00000000 */
+   -4.50359962737049600000e+15, /* 0xC3300000, 0x00000000 */
+  };
+
   int32_t j0;
   uint32_t i1, i0;
   long long int result;
@@ -96,6 +100,7 @@ __llrint (double x)
     }
 
   return sx ? -result : result;
+#endif /* ! USE_LLRINT_BUILTIN  */
 }
 
 libm_alias_double (__llrint, llrint)
