@@ -1,5 +1,4 @@
 /* s_finitel.c -- long double version of s_finite.c.
- * Conversion to IEEE quad long double by Jakub Jelinek, jj@ultra.linux.cz.
  */
 
 /*
@@ -29,8 +28,10 @@ int __finitel(_Float128 x)
 {
 	int64_t hx;
 	GET_LDOUBLE_MSW64(hx,x);
-	return (int)((u_int64_t)((hx&0x7fff000000000000LL)
-				 -0x7fff000000000000LL)>>63);
+	return (int)((uint64_t)((hx&0x7fff000000000000LL)
+				-0x7fff000000000000LL)>>63);
 }
-hidden_def (__finitel)
+mathx_hidden_def (__finitel)
+#ifndef finitel
 weak_alias (__finitel, finitel)
+#endif
